C=======================================================================C
C ATMOS.F - Part of the SOLSPEC GEANT3 application                      C
C-----------------------------------------------------------------------C
C Modification of the UCAR stdatm76.F source code available at          C
C  http://dss.ucar.edu/libraries/meteorology/                           C
C Last modification: 08/22/2002 by M. R. Moser, University of Bern, CH  C
C=======================================================================C
C
      SUBROUTINE STDZ2P(P,H1,L,TCENT,RHO)                                       
C
C COMPUTES PRESSURE, TEMPERATURE, AND DENSITY VALUES FROM INPUT HEIGHTS
C  BASED ON US STANDARD ATMOSPHERE, 1976
C  VALUES NOT VALID ABOVE 84852 KM.
C
C INPUT
C       H1    - HEIGHT IN FEET OR METERS
C       L     - UNITS FLAG - 0=METERS, 1=FEET
C OUTPUT
C       P     - PRESSURE IN MB
C       TCENT - TEMPERATURE IN DEG C
C       RHO   - DENSITY IN KG/M3
C
      DIMENSION HBASE(10),HTOP(10),TB(10),GRAD(10),ABS(10)
      DIMENSION PB(11),RB(10)
      DATA HBASE/ 0.,1.1E4,2.E4,3.2E4,4.7E4,5.1E4,7.1E4,3*84852.0/
      DATA HTOP /1.1E4,2.E4,3.2E4,4.7E4,5.1E4,7.1E4,4*84852.0/
      DATA TB / 15.,2*-56.5,-44.5,2*-2.5,-58.5,3*-86.2/
      DATA GRAD/-.0065,0.,.001,.0028,0.,-.0028,-.002,3*0./
      DATA CHECK/0./                                                            
C     H1 IS ALTITUDE, IF IN METERS SET L .LE. 0.  IF IN FEET SET L .GE. 1       
C     P IS PRESSURE IN MB.  TCENT IS TEMP CELSIUS.  RHO IS DENSITY IN KG/M**3   
C                                                                               
      IF(CHECK.NE.0.) GO TO 16                                                  
      ABSZ = 273.15                                                             
      PZERO=1013.250                                                            
      G=980.665                                                                 
      R = 83143200./28.9644                                                     
      PB(1) = PZERO                                                             
      DO 3  I=1,8                                                               
      ABS(I) = TB(I) + ABSZ                                                     
    3 CONTINUE                                                                  
       DO 15  I=1,8                                                             
      IF (GRAD(I) .EQ. 0)  GO TO 10                                             
C  HERE FOR TEMP GRADIENT NOT ZERO                                              
    5 PB(I+1) = PB(I)*((ABS(I+1)/ABS(I))**(100.*G/(-GRAD(I)*R)))                
      GO TO 12                                                                  
C  HERE FOR TEMP GRADIENT ZERO                                                  
   10 PB(I+1) = PB(I)*2.7182818**((-100.*G/(R*ABS(I)))*(HTOP(I)-HBASE(I)        
     X  ))                                                                      
   12 RB(I) = 1000.*PB(I)/(R*ABS(I)) * 1000.                                    
   15 CONTINUE                                                                  
      CHECK=10.                                                                 
   16 CONTINUE                                                                  
C                                                                               
      IF(L. LT.1) GO TO 30                                                      
C                                                                               
      H = H1*.3048                                                              
      GO TO 4                                                                   
30    H = H1                                                                    
4     CONTINUE                                                                  
C  FIND THE ATMOSPHERIC LAYER WE ARE IN                                         
   35 DO 55  I = 1,8                                                            
      IF  (H .GT. HBASE(I+1)) GO TO 55                                          
   37 IF (GRAD(I)) 39,43,39                                                     
C  TEMP GRADIENT NOT ZERO                                                       
   39 P= PB(I)*((H-HBASE(I))*GRAD(I)/ABS(I)+1)**(-100.*G/(GRAD(I)*R))           
      GO TO 45                                                                  
C  ISOTHERMAL LAYER                                                             
   43 P= PB(I)*EXP((H-HBASE(I))*(-G)/(.01*R*ABS(I)))                            
   45 T = ABS(I) + GRAD(I)*(H-HBASE(I))                                         
      TCENT = T - ABSZ                                                          
      RHO = 1.E6* P/(R*T)                                                       
      RETURN                                                                    
   55 CONTINUE                                                                  
   56 WRITE(6,98)H
98    FORMAT(' STDATM -  ALTITUDE OUT OF RANGE ',F12.2,//)
      END                                                                       
      SUBROUTINE STDP2Z(P,H,HFT,TCENT,DENS)                                     
C
C COMPUTES HEIGHT, TEMPERATURE, AND DENSITY VALUES FROM INPUT PRESSURES
C  BASED ON US STANDARD ATMOSPHERE, 1976
C  VALUES NOT VALID ABOVE 84852 KM.
C
C INPUT
C       P     - PRESSURE IN MB
C OUTPUT
C       H     - HEIGHT IN METERS
C       HFT   - HEIGHT IN FEET
C       TCENT - TEMPERATURE IN DEG C
C       DENS  - DENSITY IN KG/M3
C
      DIMENSION HBASE(10),HTOP(10),TB(10),GRAD(10),ABS(10),PB(10),RB(10)        
      DATA HBASE/ 0.,1.1E4,2.E4,3.2E4,4.7E4,5.1E4,7.1E4,3*84852.0/
      DATA HTOP /1.1E4,2.E4,3.2E4,4.7E4,5.1E4,7.1E4,4*84852.0/
      DATA GRAD/-.0065,0.,.001,.0028,0.,-.0028,-.002,3*0./
      DATA ABS/288.15,216.65,216.65,228.65,270.65,270.65,                       
     2  214.65,186.95,2*0./
      DATA PB/1013.25,0.22632064E+03,0.54748887E+02,0.86801870E+01,
     2 0.11090631E+01,0.66938874E+00,0.39564205E-01,0.37338360E-02,
     3 2*0./
      ABSZ = 273.15                                                             
      G=980.665                                                                 
      R = 83143200./28.9644                                                     
      IF(P  .GT. PB(1)) GO TO 56                                                
   35 DO 55  I = 1,8                                                            
      IF (P .GE. PB(I+1))  GO TO 37                                             
   55 CONTINUE                                                                  
   56 PRINT 1001,P                                                              
 1001 FORMAT (' STDATM- PRESSURE OUT OF EXCEEDED ',F8.2)
   37 IF (GRAD(I)) 39,43,39                                                     
C  TEMP GRADIENT NOT ZERO                                                       
   39 H = ABS(I)/(-GRAD(I)) *(1. -(P/PB(I))**(-GRAD(I)*R/(100.*G)))             
     X  + HBASE(I)                                                              
      GO TO 45                                                                  
C  ISOTHERMAL LAYER                                                             
   43 H = HBASE(I) + .01*R*ABS(I)/G *(ALOG(PB(I)/P))                            
   45 T = ABS(I) + GRAD(I)*(H-HBASE(I))                                         
      TCENT = T - ABSZ                                                          
      DENS = 1000*1000.*P/(R*T)                                                 
      HFT = H/.3048                                                             
      RETURN                                                                    
      END                                                                       
